@@ -254,6 +254,10 @@ class Base(Translation):
     def tornado_routes(self):
         return []
 
+    def prepare(self, request):
+        """this function is invoked after the module process started."""
+        self.init()
+
     def init(self):
         """this function is invoked after the module process started."""
 
@@ -478,3 +482,6 @@ class Base(Translation):
         except KeyError:
             return
         request._request_handler.reply(response)
+
+    def _is_active(self, request):
+        return request.id in self.__requests
